/*-
 * Copyright 2008 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_CAPTURE_FACTORY_H__
#define __CX88_CAPTURE_FACTORY_H__

#include "cx88_debug.h"
#include "cx88_url_capture.h"
#include "cx88_file_capture.h"
#include "cx88_audio_capture.h"

#ifdef _WITH_LINUX_COMPAT
#include "dvb/cx88_dvb_capture.h"
#include "v4l/cx88_radio_capture.h"
#endif

#include "tuner_config.h"

#define CX88_MTU_KEY "mtu"
#define CX88_BUFFER_KEY "buffer"

namespace cx88_capture_factory
{
   typedef cx88_capture* (*build_capture_func)(std::string&, const char*, tuner_config&, int&);

   struct capture_entry
   {
      const char *url_prefix;
      const char *desc;
      build_capture_func builder;
   };

   static cx88_capture *build_audio_capture(std::string &id, const char* name, tuner_config&, int &error)
   {
      return new cx88_audio_capture(id, name, error);
   }

   static cx88_capture *build_file_capture(std::string &id, const char* name, tuner_config& options, int &error)
   {
      return new cx88_file_capture(id, name, error, options.get_number<size_t>(CX88_BUFFER_KEY, CX88_DEFAULT_FILE_CACHE));
   }

   static cx88_capture *build_tcp_capture(std::string &id, const char* name, tuner_config& options, int &error)
   {
      return new cx88_url_capture(id, cx88_url_capture::CX88_URL_PROTO_TCP, name, error, options.get_number<size_t>(CX88_MTU_KEY, CX88_DEFAULT_MTU));
   }

   static cx88_capture *build_udp_capture(std::string &id, const char* name, tuner_config& options, int &error)
   {
      return new cx88_url_capture(id, cx88_url_capture::CX88_URL_PROTO_UDP, name, error, options.get_number<size_t>(CX88_MTU_KEY, CX88_DEFAULT_MTU));
   }

#ifdef _WITH_LINUX_COMPAT

   static cx88_capture *build_dvb_capture(std::string &id, const char* name, tuner_config&, int &error)
   {
      return new cx88_dvb_capture(id, name, error);
   }

   static cx88_capture *build_radio_capture(std::string &id, const char* name, tuner_config&, int &error)
   {
      return new cx88_radio_capture(id, name, error);
   }

#endif

   static const capture_entry captures[] =
   {
      {"oss://",   "Output to OSS audio device (ex: oss:///dev/dsp0)",         build_audio_capture},
      {"file://",  "Output to file (ex: file:///home/user/capture.m2t)",       build_file_capture},
#ifdef _WITH_LINUX_COMPAT
      {"dvb://",   "Create Linux DVB device nodes (ex: dvb://adapter0",        build_dvb_capture},
      {"radio://", "Create Linux V4L2 radio device nodes (ex: radio://radio0", build_radio_capture},
#endif
      {"tcp://",   "Output to TCP client socket (ex: tcp://localhost:8802)",   build_tcp_capture},
      {"udp://",   "Output to UDP port (ex: udp://localhost:8802)",            build_udp_capture},
   };

   static cx88_capture *create(
      const char *name,
      tuner_config &options,
      int &error)
   {
      std::string id;
      try
      {
         id = name;
      }
      catch (...)
      {
         error = ENOMEM;
         return NULL;
      }
      for (size_t i = 0; i < (sizeof(captures) / sizeof(captures[0])); ++i)
      {
         size_t prefix_len = strlen(captures[i].url_prefix);
         if (strncasecmp(name, captures[i].url_prefix, prefix_len) == 0)
         {
            return captures[i].builder(id, name + prefix_len, options, error);
         }
      }
      CX88ERR << "Unrecognized capture URL " << name << std::endl;
      error = EINVAL;
      return NULL;
   }
};

#endif
