/*-
 * Copyright 2008 Konstantin Dimitrov <kosio.dimitrov@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/errno.h>
#include "fmd1216me.h"
#include "cx22702.h"
#include "cx88_debug.h"
#include "wintv_hvr4000.h"

static int hvr4000_gpio(int fd)
{
   int error = 0;
   cx88_common_gpio gpio;
   gpio.in_write_mask = 0x80;
   gpio.in_read_mask = 0x0;
   gpio.inout_gpio = 0x80;
   if ((error = ioctl(fd, CX88_COMMON_GPIO, &gpio)) != 0)
   {
      CX88ERR << "Error initializing CX22702 GPIO: " << strerror(errno) << std::endl;
      return error;
   }
   usleep(1000);
   gpio.in_write_mask = 0x80;
   gpio.in_read_mask = 0x0;
   gpio.inout_gpio = 0x0;
   if ((error = ioctl(fd, CX88_COMMON_GPIO, &gpio)) != 0)
   {
      CX88ERR << "Error clearing CX22702 GPIO: " << strerror(errno) << std::endl;
      return error;
   }
   usleep(50);
   gpio.in_write_mask = 0x80;
   gpio.in_read_mask = 0x0;
   gpio.inout_gpio = 0x80;
   if ((error = ioctl(fd, CX88_COMMON_GPIO, &gpio)) != 0)
   {
      CX88ERR << "Error re-initializing CX22702 GPIO: " << strerror(errno) << std::endl;
      return error;
   }
   usleep(1000);
   gpio.in_write_mask = 0x4;
   gpio.in_read_mask = 0x0;
   gpio.inout_gpio = 0x0;
   if ((error = (ioctl(fd, CX88_COMMON_GPIO, &gpio)) != 0))
   {
      CX88ERR << "Error clearing CX23416 GPIO: " << strerror(errno) << std::endl;
      return error;
   }
   usleep(1000);
   return error;
}

namespace cx88_mpeg_driver_factory
{

   int wintv_hvr4000::create_i2c_drivers(void)
   {
      int error = hvr4000_gpio(m_fd);
      if (error)
      {
         return error;
      }
      uint32_t reset_val = 0;
      error = ioctl(m_fd, CX88_COMMON_PERIPH_RESET, &reset_val);
      if (error)
      {
         CX88ERR << "Error resetting peripheral devices on WinTV HVR-4000" << std::endl;
         return error;
      }
      m_i2c_devs[1].driver = new cx22702(m_tunerconf, *(m_i2c_devs[1].dev), DVB_INPUT_SERIAL, error);
      m_i2c_devs[0].driver = new fmd1216me(m_tunerconf, *(m_i2c_devs[0].dev));
      if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL))
      {
         error = ENOMEM;
      }
      if (error)
      {
         CX88ERR << "Error initializing tuner/demodulator driver for WinTV HVR-4000" << std::endl;
         return error;
      }
      return 0;
   }

   void wintv_hvr4000::destroy_i2c_drivers(void)
   {
      delete m_i2c_devs[0].driver;
      delete m_i2c_devs[1].driver;  
   }

   int wintv_hvr4000::create_i2c_devices(void)
   {
      m_i2c_devs[0].conf.in_addr = 0x61;
      m_i2c_devs[1].conf.in_addr = 0x63;
      return 0;
   }
}

namespace cx88_analog_driver_factory
{

   const cx88_source wintv_hvr4000::m_sources[] =
      {{"Tuner",     "RF input",                  cx88_driver::CX88_SOURCE_DEFAULT},
       {"SVideo",    "S-video input",             wintv_hvr4000::HVR4000_SOURCE_SVIDEO},
       {"Composite", "Composite video input",     wintv_hvr4000::HVR4000_SOURCE_COMPOSITE},
       {NULL,        NULL,                        0}};

   int wintv_hvr4000::create_i2c_drivers(void)
   {
      int error = hvr4000_gpio(m_fd);
      if (error)
      {
         return error;
      }
      m_i2c_devs[1].driver = new cx22702(m_tunerconf, *(m_i2c_devs[1].dev), DVB_INPUT_SERIAL, error);
      m_i2c_devs[0].driver = new fmd1216me(m_tunerconf, *(m_i2c_devs[0].dev));
      if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL))
      {
         error = ENOMEM;
      }
      if (error)
      {
         CX88ERR << "Error initializing tuner driver for WinTV HVR-4000" << std::endl;
      }
      return error;
   }

   void wintv_hvr4000::destroy_i2c_drivers(void)
   {
      delete m_i2c_devs[0].driver;
      delete m_i2c_devs[1].driver;
   }

   int wintv_hvr4000::create_i2c_devices(void)
   {
      m_i2c_devs[0].conf.in_addr = 0x61;
      m_i2c_devs[0].source_mask = CX88_SOURCE_DEFAULT;
      m_i2c_devs[1].conf.in_addr = 0x63;
      m_i2c_devs[1].source_mask = 0;
      return 0;
   }

   int wintv_hvr4000::select_source(uint32_t source_mask)
   {
      switch (source_mask)
      {
         case CX88_SOURCE_DEFAULT:
            m_input_mux = CX88_VIDEO_INPUT_MUX1;
            m_input_type = CX88_VIDEO_INPUT_COMPOSITE;
            break;
         case HVR4000_SOURCE_SVIDEO:
            m_input_mux = CX88_VIDEO_INPUT_MUX3;
            m_input_type = CX88_VIDEO_INPUT_SVIDEO;
            break;
         case HVR4000_SOURCE_COMPOSITE:
            m_input_mux = CX88_VIDEO_INPUT_MUX2;
            m_input_type = CX88_VIDEO_INPUT_COMPOSITE;
            break;
         default:
            return EINVAL;
      }
      return 0;
   }

   int wintv_hvr4000::set_capture_options(cx88_video_capture &capture)
   {
      capture.in_input_source = m_input_mux;
      capture.in_input_type = m_input_type;
      return 0;
   }
}

